"""Concurrency and race condition tests.

Tests concurrent operations to ensure thread-safety and absence of race conditions.
"""

from __future__ import annotations

import asyncio
from pathlib import Path

import pytest

from mcp_common.cli.factory import write_runtime_health, load_runtime_health
from mcp_common.config import MCPBaseSettings
from mcp_common.security.sanitization import sanitize_output
from mcp_common.security.api_keys import validate_api_key_format


class TestConcurrency:
    """Test concurrent operations don't cause race conditions."""

    @pytest.mark.asyncio
    async def test_concurrent_health_snapshot_writes(self, tmp_path: Path) -> None:
        """Test concurrent writes to health snapshot are safe."""
        snapshot_path = tmp_path / "health.json"
        from mcp_common.cli import RuntimeHealthSnapshot

        # Create a snapshot
        snapshot = RuntimeHealthSnapshot(orchestrator_pid=12345)

        # Write it once first
        write_runtime_health(snapshot_path, snapshot)

        # Create multiple concurrent read tasks
        tasks = [
            asyncio.to_thread(load_runtime_health, snapshot_path)
            for _ in range(10)
        ]

        # All reads should succeed and return valid data
        results = await asyncio.gather(*tasks)

        # All results should be valid
        assert all(r is not None for r in results)
        assert all(r.orchestrator_pid == 12345 for r in results)

    @pytest.mark.asyncio
    async def test_concurrent_config_loading(self, tmp_path: Path) -> None:
        """Test concurrent config loading doesn't cause issues."""
        (tmp_path / "settings").mkdir()

        # Create 10 concurrent config loads
        tasks = []
        for i in range(10):
            async def load_config() -> object:
                class TestSettings(MCPBaseSettings):
                    test_field: str = "default"

                return TestSettings.load("concurrent-test")

            tasks.append(load_config())

        results = await asyncio.gather(*tasks)
        assert all(r.test_field == "default" for r in results)  # type: ignore

    @pytest.mark.asyncio
    async def test_concurrent_sanitization(self) -> None:
        """Test concurrent sanitization calls don't interfere."""
        data = {"api_key": "sk-test-key", "user": "john"}

        # Sanitize same data concurrently
        tasks = [asyncio.to_thread(sanitize_output, data) for _ in range(100)]
        results = await asyncio.gather(*tasks)

        # All results should be identical
        assert all(r == results[0] for r in results)

    @pytest.mark.asyncio
    async def test_concurrent_api_key_validation(self) -> None:
        """Test concurrent API key validation is thread-safe."""
        key = "sk-test-api-key-12345678"

        # Validate same key concurrently
        tasks = [
            asyncio.to_thread(validate_api_key_format, key, provider="generic")
            for _ in range(100)
        ]
        results = await asyncio.gather(*tasks)

        # All results should be identical
        assert all(r == results[0] for r in results)

    @pytest.mark.asyncio
    async def test_concurrent_nested_sanitization(self) -> None:
        """Test concurrent sanitization of nested data structures."""
        data = {
            "level1": {
                "level2": {
                    "api_key": "sk-test-key"
                }
            }
        }

        # Sanitize same nested data concurrently
        tasks = [asyncio.to_thread(sanitize_output, data) for _ in range(50)]
        results = await asyncio.gather(*tasks)

        # All results should be identical
        assert all(r == results[0] for r in results)

    @pytest.mark.asyncio
    async def test_concurrent_list_sanitization(self) -> None:
        """Test concurrent sanitization of lists."""
        data = [
            {"api_key": f"sk-test-key-{i}", "user": f"user{i}"}
            for i in range(10)
        ]

        # Sanitize same list concurrently
        tasks = [asyncio.to_thread(sanitize_output, data) for _ in range(50)]
        results = await asyncio.gather(*tasks)

        # All results should be identical
        assert all(str(r) == str(results[0]) for r in results)

    @pytest.mark.asyncio
    async def test_concurrent_mixed_data_sanitization(self) -> None:
        """Test concurrent sanitization of mixed data types."""
        data = {
            "string": "test",
            "number": 42,
            "list": [1, 2, 3],
            "dict": {"nested": "value"},
            "bool": True,
            "none": None,
        }

        # Sanitize same mixed data concurrently
        tasks = [asyncio.to_thread(sanitize_output, data) for _ in range(50)]
        results = await asyncio.gather(*tasks)

        # All results should be identical
        assert all(str(r) == str(results[0]) for r in results)

    @pytest.mark.asyncio
    async def test_concurrent_different_keys(self) -> None:
        """Test concurrent validation of different keys doesn't interfere."""
        keys = [f"sk-test-key-{i:04d}" for i in range(10)]

        # Validate different keys concurrently
        tasks = [
            asyncio.to_thread(validate_api_key_format, key, provider="generic")
            for key in keys
            for _ in range(10)  # Repeat each key 10 times
        ]
        results = await asyncio.gather(*tasks)

        # All should succeed
        assert all(isinstance(r, str) for r in results)

    @pytest.mark.asyncio
    async def test_concurrent_empty_sanitization(self) -> None:
        """Test concurrent sanitization of empty/None values."""
        test_cases = [
            {},
            [],
            "",
            None,
            {"empty": ""},
            {"nested": {"empty": []}},
        ]

        # Sanitize various empty values concurrently
        tasks = [asyncio.to_thread(sanitize_output, case) for case in test_cases for _ in range(10)]
        results = await asyncio.gather(*tasks)

        # All should complete without error (None is a valid return for None input)
        assert len(results) == len(test_cases) * 10

        # Verify results match expectations (None -> None, others -> same type)
        for i, case in enumerate(test_cases):
            expected = None if case is None else case
            # All 10 repetitions of this case should have same result
            for j in range(10):
                result = results[i * 10 + j]
                assert type(result) == type(expected)

    @pytest.mark.asyncio
    async def test_concurrent_large_sanitization(self) -> None:
        """Test concurrent sanitization of large data structures."""
        large_data = {
            f"key_{i}": f"value_{i}" * 100
            for i in range(100)
        }

        # Sanitize large data concurrently
        tasks = [asyncio.to_thread(sanitize_output, large_data) for _ in range(20)]
        results = await asyncio.gather(*tasks)

        # All results should be identical
        assert all(str(r) == str(results[0]) for r in results)



"""Tests for ValidationMixin configuration validation patterns."""

from __future__ import annotations

import importlib
from unittest.mock import Mock

import pytest
from pydantic import BaseModel, ValidationError

from mcp_common.config import ValidationMixin
from mcp_common.config.validation_mixin import ValidationMixin as VM
from mcp_common.exceptions import (
    CredentialValidationError,
    ServerConfigurationError,
)


@pytest.mark.unit
class TestValidationMixinRequiredField:
    """Tests for validate_required_field method."""

    def test_valid_field_passes(self) -> None:
        """Test validation passes with valid non-empty field."""
        VM.validate_required_field("username", "testuser")
        VM.validate_required_field("api_key", "sk-1234567890")

    def test_none_field_fails(self) -> None:
        """Test validation fails with None field."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_required_field("username", None)

        assert "username is not set in configuration" in str(exc_info.value)
        assert exc_info.value.field == "username"

    def test_empty_string_fails(self) -> None:
        """Test validation fails with empty string."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_required_field("password", "")

        assert "password is not set in configuration" in str(exc_info.value)

    def test_whitespace_only_fails(self) -> None:
        """Test validation fails with whitespace-only string."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_required_field("api_key", "   ")

        assert "api_key is not set in configuration" in str(exc_info.value)

    def test_with_context(self) -> None:
        """Test validation includes context in error message."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_required_field(
                "host", None, context="Database Controller"
            )

        assert "Database Controller host is not set" in str(exc_info.value)

    def test_fallback_to_valueerror_without_exceptions(self, monkeypatch: pytest.MonkeyPatch) -> None:
        """Test fallback to ValueError when exceptions unavailable."""
        # Force exceptions to be unavailable
        monkeypatch.setattr(
            "mcp_common.config.validation_mixin.EXCEPTIONS_AVAILABLE", False
        )

        with pytest.raises(ValueError) as exc_info:
            VM.validate_required_field("field", None)

        assert "field is not set in configuration" in str(exc_info.value)


@pytest.mark.unit
class TestValidationMixinMinLength:
    """Tests for validate_min_length method."""

    def test_valid_length_passes(self) -> None:
        """Test validation passes when length meets minimum."""
        VM.validate_min_length("password", "securepassword123", min_length=12)
        VM.validate_min_length("api_key", "sk-1234567890", min_length=10)

    def test_short_value_fails(self) -> None:
        """Test validation fails when value is too short."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_min_length("password", "short", min_length=12)

        assert "password is too short" in str(exc_info.value)
        assert "Required: 12 characters, got: 5" in str(exc_info.value)
        assert exc_info.value.field == "password"

    def test_with_context(self) -> None:
        """Test validation includes context in error message."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_min_length(
                "api_key", "short", min_length=20, context="API"
            )

        assert "API api_key is too short" in str(exc_info.value)

    def test_exactly_min_length_passes(self) -> None:
        """Test value exactly at minimum length passes."""
        VM.validate_min_length("field", "abc", min_length=3)


@pytest.mark.unit
class TestValidationMixinCredentials:
    """Tests for validate_credentials method."""

    def test_valid_credentials_pass(self) -> None:
        """Test validation with valid username and password."""
        class TestModel(BaseModel, ValidationMixin):
            pass

        model = TestModel()
        model.validate_credentials(
            username="testuser",
            password="securepassword123",
        )

    def test_missing_username_fails(self) -> None:
        """Test validation fails when username is missing."""
        class TestModel(BaseModel, ValidationMixin):
            pass

        model = TestModel()

        with pytest.raises(CredentialValidationError) as exc_info:
            model.validate_credentials(username=None, password="password123")

        assert "username is not set" in str(exc_info.value)

    def test_missing_password_fails(self) -> None:
        """Test validation fails when password is missing."""
        class TestModel(BaseModel, ValidationMixin):
            pass

        model = TestModel()

        with pytest.raises(CredentialValidationError) as exc_info:
            model.validate_credentials(username="testuser", password=None)

        assert "password is not set" in str(exc_info.value)

    def test_short_password_fails(self) -> None:
        """Test validation fails when password is too short."""
        class TestModel(BaseModel, ValidationMixin):
            pass

        model = TestModel()

        with pytest.raises(CredentialValidationError) as exc_info:
            model.validate_credentials(
                username="testuser",
                password="short",
                min_password_length=12,
            )

        assert "password is too short" in str(exc_info.value)

    def test_custom_min_password_length(self) -> None:
        """Test validation with custom minimum password length."""
        class TestModel(BaseModel, ValidationMixin):
            pass

        model = TestModel()

        # Should pass with 8 chars
        model.validate_credentials(
            username="testuser",
            password="pass1234",
            min_password_length=8,
        )

        # Should fail with 7 chars
        with pytest.raises(CredentialValidationError):
            model.validate_credentials(
                username="testuser",
                password="pass123",
                min_password_length=8,
            )

    def test_with_context(self) -> None:
        """Test validation includes context in error message."""
        class TestModel(BaseModel, ValidationMixin):
            pass

        model = TestModel()

        with pytest.raises(CredentialValidationError) as exc_info:
            model.validate_credentials(
                username=None,
                password="password",
                context="Database",
            )

        assert "Database username is not set" in str(exc_info.value)


@pytest.mark.unit
class TestValidationMixinURLParts:
    """Tests for validate_url_parts method."""

    def test_valid_host_passes(self) -> None:
        """Test validation passes with valid host."""
        VM.validate_url_parts(host="localhost")
        VM.validate_url_parts(host="api.example.com")
        VM.validate_url_parts(host="192.168.1.1")

    def test_missing_host_fails(self) -> None:
        """Test validation fails when host is missing."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_url_parts(host=None)

        assert "host is not set in configuration" in str(exc_info.value)
        assert exc_info.value.field == "host"

    def test_empty_host_fails(self) -> None:
        """Test validation fails when host is empty."""
        with pytest.raises(ServerConfigurationError):
            VM.validate_url_parts(host="   ")

    def test_valid_port_passes(self) -> None:
        """Test validation passes with valid port."""
        VM.validate_url_parts(host="localhost", port=8000)
        VM.validate_url_parts(host="localhost", port=1)
        VM.validate_url_parts(host="localhost", port=65535)

    def test_port_out_of_range_high(self) -> None:
        """Test validation fails when port > 65535."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_url_parts(host="localhost", port=65536)

        assert "port must be between 1 and 65535" in str(exc_info.value)

    def test_port_out_of_range_low(self) -> None:
        """Test validation fails when port < 1."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_url_parts(host="localhost", port=0)

        assert "port must be between 1 and 65535" in str(exc_info.value)

    def test_port_wrong_type(self) -> None:
        """Test validation fails with non-integer port."""
        with pytest.raises(ServerConfigurationError):
            VM.validate_url_parts(host="localhost", port="8080")  # type: ignore[arg-type]

    def test_none_port_passes(self) -> None:
        """Test validation passes with None port (optional)."""
        VM.validate_url_parts(host="localhost", port=None)

    def test_with_context(self) -> None:
        """Test validation includes context in error message."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_url_parts(host=None, context="Redis")

        assert "Redis host is not set" in str(exc_info.value)


@pytest.mark.unit
class TestValidationMixinOneOfRequired:
    """Tests for validate_one_of_required method."""

    def test_one_value_set_passes(self) -> None:
        """Test validation passes when at least one field is set."""
        VM.validate_one_of_required(
            field_names=["host", "url"],
            values=["localhost", None],
        )

        VM.validate_one_of_required(
            field_names=["host", "url"],
            values=[None, "https://api.example.com"],
        )

    def test_all_values_set_passes(self) -> None:
        """Test validation passes when all fields are set."""
        VM.validate_one_of_required(
            field_names=["host", "url"],
            values=["localhost", "https://api.example.com"],
        )

    def test_no_values_set_fails(self) -> None:
        """Test validation fails when all fields are None/empty."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_one_of_required(
                field_names=["host", "url"],
                values=[None, None],
            )

        assert "At least one of [host, url] is required" in str(exc_info.value)
        assert exc_info.value.field == "multiple_fields"

    def test_empty_strings_not_counted(self) -> None:
        """Test validation fails with only empty strings."""
        with pytest.raises(ServerConfigurationError):
            VM.validate_one_of_required(
                field_names=["field1", "field2"],
                values=["  ", "  "],
            )

    def test_with_context(self) -> None:
        """Test validation includes context in error message."""
        with pytest.raises(ServerConfigurationError) as exc_info:
            VM.validate_one_of_required(
                field_names=["option_a", "option_b"],
                values=[None, None],
                context="Configuration",
            )

        assert "Configuration: At least one of [option_a, option_b] is required" in str(
            exc_info.value
        )

    def test_mismatched_lengths_raise_valueerror(self) -> None:
        """Test ValueError when field_names and values lengths differ."""
        with pytest.raises(ValueError, match="field_names and values must have same length"):
            VM.validate_one_of_required(
                field_names=["field1", "field2"],
                values=["value1"],
            )


@pytest.mark.unit
class TestValidationMixinIntegration:
    """Integration tests for ValidationMixin with real models."""

    def test_model_with_validation_mixin(self) -> None:
        """Test ValidationMixin works with Pydantic models."""

        class ServerSettings(BaseModel, ValidationMixin):
            username: str
            password: str
            host: str
            port: int | None = None

        settings = ServerSettings(
            username="admin",
            password="securepassword123",
            host="localhost",
        )

        # Validate credentials
        settings.validate_credentials(
            username=settings.username,
            password=settings.password,
        )

        # Validate URL parts
        settings.validate_url_parts(
            host=settings.host,
            port=settings.port,
        )

    def test_model_validation_chain(self) -> None:
        """Test multiple validations in sequence."""

        class AppConfig(BaseModel, ValidationMixin):
            api_key: str
            api_secret: str
            endpoint: str

        config = AppConfig(
            api_key="key-1234567890",
            api_secret="secret-1234567890",
            endpoint="https://api.example.com",
        )

        # Chain validations
        config.validate_required_field("api_key", config.api_key)
        config.validate_required_field("api_secret", config.api_secret)
        config.validate_min_length("api_key", config.api_key, min_length=10)
        config.validate_url_parts(host=config.endpoint)



"""Tests for prompt-toolkit backend."""

from __future__ import annotations

from unittest.mock import AsyncMock, Mock, patch

import pytest

from mcp_common.backends.toolkit import PromptToolkitBackend
from mcp_common.prompting.exceptions import BackendUnavailableError, DialogDisplayError
from mcp_common.prompting.models import DialogResult, NotificationLevel, PromptAdapterSettings, PromptConfig


@pytest.mark.unit
class TestPromptToolkitBackend:
    """Tests for PromptToolkitBackend."""

    def test_backend_initialization(self) -> None:
        """Test backend initializes with config."""
        config = PromptAdapterSettings(timeout=30)
        backend = PromptToolkitBackend(config=config)
        assert backend.config == config
        assert backend.backend_name == "prompt-toolkit"

    def test_backend_name_property(self) -> None:
        """Test backend_name property."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())
        assert backend.backend_name == "prompt-toolkit"

    def test_is_available_static(self) -> None:
        """Test is_available_static method."""
        # prompt-toolkit should be available in test environment
        assert PromptToolkitBackend.is_available_static() is True

    def test_is_available_instance(self) -> None:
        """Test is_available instance method."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())
        assert backend.is_available() is True

    def test_unavailable_error_when_not_installed(self) -> None:
        """Test error when prompt-toolkit is not installed."""
        with patch(
            "mcp_common.backends.toolkit.PROMPT_TOOLKIT_AVAILABLE",
            False,
        ):
            with pytest.raises(BackendUnavailableError) as exc_info:
                PromptToolkitBackend(config=PromptAdapterSettings())

            error = exc_info.value
            assert error.backend == "prompt-toolkit"
            assert "install" in str(error).lower()


@pytest.mark.unit
class TestPromptToolkitBackendAlert:
    """Tests for alert method."""

    @pytest.mark.asyncio
    async def test_alert_basic(self) -> None:
        """Test basic alert dialog."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("builtins.input", return_value=""):
            with patch("builtins.print"):
                result = await backend.alert(
                    title="Test Title",
                    message="Test message",
                )

                assert isinstance(result, DialogResult)
                assert result.button_clicked == "OK"
                assert result.cancelled is False

    @pytest.mark.asyncio
    async def test_alert_with_buttons(self) -> None:
        """Test alert with custom buttons."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            # Simulate user selecting "Cancel" (option 2)
            mock_prompt.return_value = "2"
            result = await backend.alert(
                title="Confirm",
                message="Are you sure?",
                buttons=["OK", "Cancel"],
            )

            assert result.button_clicked == "Cancel"
            assert result.cancelled is False

    @pytest.mark.asyncio
    async def test_alert_with_detail(self) -> None:
        """Test alert with detail text."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("builtins.input", return_value=""):
            with patch("builtins.print"):
                result = await backend.alert(
                    title="Error",
                    message="Something went wrong",
                    detail="Detailed error information",
                )

                assert result.button_clicked == "OK"
                assert result.cancelled is False

    @pytest.mark.asyncio
    async def test_alert_cancelled(self) -> None:
        """Test alert when user cancels."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("builtins.input", side_effect=KeyboardInterrupt()):
            with patch("builtins.print"):
                result = await backend.alert(
                    title="Test",
                    message="Test message",
                )

                assert result.cancelled is True

    @pytest.mark.asyncio
    async def test_alert_handles_exception(self) -> None:
        """Test alert method handles exceptions gracefully."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        # Mock prompt_choice to raise an exception
        with patch.object(backend, "prompt_choice", side_effect=Exception("Unexpected error")):
            with pytest.raises(DialogDisplayError) as exc_info:
                await backend.alert(
                    title="Test",
                    message="Test message",
                    buttons=["OK", "Cancel"],
                )

            assert exc_info.value.backend == "prompt-toolkit"
            assert exc_info.value.dialog_type == "alert"


@pytest.mark.unit
class TestPromptToolkitBackendConfirm:
    """Tests for confirm method."""

    @pytest.mark.asyncio
    async def test_confirm_yes(self) -> None:
        """Test confirm dialog returns True."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.confirm") as mock_confirm:
            with patch("builtins.print"):
                mock_confirm.return_value = True
                result = await backend.confirm(
                    title="Confirm",
                    message="Do you want to continue?",
                )

                assert result is True

    @pytest.mark.asyncio
    async def test_confirm_no(self) -> None:
        """Test confirm dialog returns False."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.confirm") as mock_confirm:
            with patch("builtins.print"):
                mock_confirm.return_value = False
                result = await backend.confirm(
                    title="Confirm",
                    message="Do you want to continue?",
                )

                assert result is False

    @pytest.mark.asyncio
    async def test_confirm_with_custom_labels(self) -> None:
        """Test confirm with custom button labels."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.confirm") as mock_confirm:
            with patch("builtins.print"):
                mock_confirm.return_value = True
                result = await backend.confirm(
                    title="Deploy",
                    message="Deploy to production?",
                    yes_label="Deploy",
                    no_label="Cancel",
                )

                assert result is True

    @pytest.mark.asyncio
    async def test_confirm_with_default_true(self) -> None:
        """Test confirm with default True."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.confirm") as mock_confirm:
            with patch("builtins.print"):
                mock_confirm.return_value = True
                result = await backend.confirm(
                    title="Confirm",
                    message="Continue?",
                    default=True,
                )

                assert result is True

    @pytest.mark.asyncio
    async def test_confirm_handles_exception(self) -> None:
        """Test confirm method handles exceptions gracefully."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.confirm", side_effect=Exception("Unexpected error")):
            # Should return False (safe default) on exception
            result = await backend.confirm(
                title="Test",
                message="Test message",
            )

            # Should return False (safe default)
            assert result is False


@pytest.mark.unit
class TestPromptToolkitBackendPromptText:
    """Tests for prompt_text method."""

    @pytest.mark.asyncio
    async def test_prompt_text_basic(self) -> None:
        """Test basic text prompt."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            mock_prompt.return_value = "Hello World"
            result = await backend.prompt_text(
                title="Input",
                message="Enter text:",
            )

            assert result == "Hello World"

    @pytest.mark.asyncio
    async def test_prompt_text_with_default(self) -> None:
        """Test text prompt with default value."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            mock_prompt.return_value = "default_value"
            result = await backend.prompt_text(
                title="Input",
                message="Enter text:",
                default="default_value",
            )

            assert result == "default_value"

    @pytest.mark.asyncio
    async def test_prompt_text_cancelled(self) -> None:
        """Test text prompt when cancelled."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            # Simulate Ctrl+C
            mock_prompt.side_effect = KeyboardInterrupt()
            result = await backend.prompt_text(
                title="Input",
                message="Enter text:",
            )

            assert result is None

    @pytest.mark.asyncio
    async def test_prompt_text_secure(self) -> None:
        """Test secure text prompt (password)."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            mock_prompt.return_value = "secret_password"
            result = await backend.prompt_text(
                title="Password",
                message="Enter password:",
                secure=True,
            )

            assert result == "secret_password"


@pytest.mark.unit
class TestPromptToolkitBackendPromptChoice:
    """Tests for prompt_choice method."""

    @pytest.mark.asyncio
    async def test_prompt_choice_basic(self) -> None:
        """Test basic choice prompt."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            with patch("builtins.print"):
                # Simulate user selecting "Option B" by entering "2"
                mock_prompt.return_value = "2"
                result = await backend.prompt_choice(
                    title="Select",
                    message="Choose an option:",
                    choices=["Option A", "Option B", "Option C"],
                )

                assert result == "Option B"

    @pytest.mark.asyncio
    async def test_prompt_choice_with_default(self) -> None:
        """Test choice prompt with default selection."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            with patch("builtins.print"):
                # Empty input should return default
                mock_prompt.return_value = ""
                result = await backend.prompt_choice(
                    title="Select",
                    message="Choose:",
                    choices=["A", "B", "C"],
                    default="B",
                )

                assert result == "B"

    @pytest.mark.asyncio
    async def test_prompt_choice_cancelled(self) -> None:
        """Test choice prompt when cancelled."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            with patch("builtins.print"):
                # Simulate Ctrl+C
                mock_prompt.side_effect = KeyboardInterrupt()
                result = await backend.prompt_choice(
                    title="Select",
                    message="Choose:",
                    choices=["A", "B"],
                )

                assert result is None


@pytest.mark.unit
class TestPromptToolkitBackendNotify:
    """Tests for notify method."""

    @pytest.mark.asyncio
    async def test_notify_info(self) -> None:
        """Test info notification."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("builtins.print") as mock_print:
            result = await backend.notify(
                title="Info",
                message="Information message",
                level=NotificationLevel.INFO,
            )

            assert result is True
            mock_print.assert_called_once()
            args = mock_print.call_args[0][0]
            assert "Info" in args
            assert "Information message" in args

    @pytest.mark.asyncio
    async def test_notify_warning(self) -> None:
        """Test warning notification."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("builtins.print") as mock_print:
            result = await backend.notify(
                title="Warning",
                message="Warning message",
                level=NotificationLevel.WARNING,
            )

            assert result is True
            args = mock_print.call_args[0][0]
            assert "⚠" in args or "Warning" in args

    @pytest.mark.asyncio
    async def test_notify_error(self) -> None:
        """Test error notification."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("builtins.print") as mock_print:
            result = await backend.notify(
                title="Error",
                message="Error message",
                level=NotificationLevel.ERROR,
            )

            assert result is True
            args = mock_print.call_args[0][0]
            assert "Error" in args

    @pytest.mark.asyncio
    async def test_notify_success(self) -> None:
        """Test success notification."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("builtins.print") as mock_print:
            result = await backend.notify(
                title="Success",
                message="Operation completed",
                level=NotificationLevel.SUCCESS,
            )

            assert result is True
            args = mock_print.call_args[0][0]
            assert "Success" in args or "✓" in args or "✅" in args


@pytest.mark.unit
class TestPromptToolkitBackendFileSelection:
    """Tests for file and directory selection methods."""

    @pytest.mark.asyncio
    async def test_select_file_single(self) -> None:
        """Test single file selection."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            mock_prompt.return_value = "/path/to/file.txt"
            result = await backend.select_file(
                title="Select File",
                allowed_types=["txt", "md"],
                multiple=False,
            )

            assert result is not None
            assert len(result) == 1
            assert str(result[0]) == "/path/to/file.txt"

    @pytest.mark.asyncio
    async def test_select_file_multiple(self) -> None:
        """Test multiple file selection."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            # Simulate comma-separated paths
            mock_prompt.return_value = "/path/file1.txt, /path/file2.txt"
            result = await backend.select_file(
                title="Select Files",
                allowed_types=None,
                multiple=True,
            )

            assert result is not None
            assert len(result) == 2
            assert str(result[0]) == "/path/file1.txt"
            assert str(result[1]) == "/path/file2.txt"

    @pytest.mark.asyncio
    async def test_select_file_cancelled(self) -> None:
        """Test file selection when cancelled."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            mock_prompt.side_effect = KeyboardInterrupt()
            result = await backend.select_file(
                title="Select File",
            )

            assert result is None

    @pytest.mark.asyncio
    async def test_select_directory(self) -> None:
        """Test directory selection."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            mock_prompt.return_value = "/path/to/directory"
            result = await backend.select_directory(
                title="Select Directory",
            )

            assert result is not None
            assert str(result) == "/path/to/directory"

    @pytest.mark.asyncio
    async def test_select_directory_cancelled(self) -> None:
        """Test directory selection when cancelled."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.prompt") as mock_prompt:
            mock_prompt.side_effect = KeyboardInterrupt()
            result = await backend.select_directory(
                title="Select Directory",
            )

            assert result is None


@pytest.mark.unit
class TestPromptToolkitBackendErrorHandling:
    """Tests for error handling in prompt-toolkit backend."""

    @pytest.mark.asyncio
    async def test_confirm_handles_exception(self) -> None:
        """Test confirm method handles exceptions gracefully."""
        backend = PromptToolkitBackend(config=PromptAdapterSettings())

        with patch("mcp_common.backends.toolkit.confirm", side_effect=Exception("Unexpected error")):
            # Should return False (safe default) on exception
            result = await backend.confirm(
                title="Test",
                message="Test message",
            )

            # Should return False (safe default)
            assert result is False



"""
Code Graph Analyzer - Shared infrastructure for Session Buddy and Mahavishnu.

Analyzes codebase structure to extract functions, classes, imports, and call graphs.
Used for context compaction, RAG enhancement, and workflow intelligence.

Version: 0.4.0
"""

from __future__ import annotations

import ast
import operator
import time
from collections import defaultdict
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any


@dataclass
class CodeNode:
    """Base class for code nodes in the graph."""

    id: str
    name: str
    file_id: str
    node_type: str  # "file", "function", "class", "import"


@dataclass
class FunctionNode(CodeNode):
    """Represents a function or method in the codebase."""

    is_export: bool
    start_line: int
    end_line: int
    calls: list[str] = field(default_factory=list)
    docstring: str | None = None
    complexity: int = 1  # Cyclomatic complexity


@dataclass
class ClassNode(CodeNode):
    """Represents a class in the codebase."""

    start_line: int
    end_line: int
    methods: list[str] = field(default_factory=list)
    base_classes: list[str] = field(default_factory=list)
    docstring: str | None = None


@dataclass
class ImportNode(CodeNode):
    """Represents an import statement."""

    module: str
    alias: str | None = None
    is_from_import: bool = False
    imported_names: list[str] = field(default_factory=list)


@dataclass
class FileNode(CodeNode):
    """Represents a source file in the codebase."""

    path: str
    language: str
    functions: list[str] = field(default_factory=list)
    classes: list[str] = field(default_factory=list)
    imports: list[str] = field(default_factory=list)


class CodeGraphAnalyzer:
    """
    Analyze and index codebase structure.

    Extracts functions, classes, imports, and builds a call graph.
    Supports Python initially, with planned support for JS/TS.

    Example:
        ```python
        analyzer = CodeGraphAnalyzer(Path("/path/to/repo"))
        stats = await analyzer.analyze_repository("/path/to/repo")

        # Get function context
        context = await analyzer.get_function_context("my_function")

        # Find related files
        related = await analyzer.find_related_files("src/main.py", "imports")
        ```
    """

    def __init__(self, project_path: Path | str):
        """Initialize the analyzer with a project path.

        Args:
            project_path: Path to the project directory to analyze
        """
        self.project_path = Path(project_path)
        self.nodes: dict[str, CodeNode] = {}
        self._call_graph: dict[str, set[str]] = defaultdict(set)
        self._import_graph: dict[str, set[str]] = defaultdict(set)
        self._file_nodes: dict[str, FileNode] = {}

    async def analyze_repository(
        self,
        repo_path: str,
        languages: list[str] | None = None,
        include_tests: bool = False,
    ) -> dict[str, Any]:
        """
        Analyze a repository and build the code graph.

        Args:
            repo_path: Path to the repository
            languages: Languages to index (default: ["python"])
            include_tests: Whether to include test files

        Returns:
            Statistics about the indexed code:
            {
                "files_indexed": int,
                "functions_indexed": int,
                "classes_indexed": int,
                "calls_indexed": int,
                "imports_indexed": int,
                "duration_ms": int
            }
        """
        start_time = time.time()
        repo = Path(repo_path)

        if not repo.exists():
            raise FileNotFoundError(f"Repository not found: {repo_path}")

        languages = languages or ["python"]
        stats = {
            "files_indexed": 0,
            "functions_indexed": 0,
            "classes_indexed": 0,
            "calls_indexed": 0,
            "imports_indexed": 0,
            "duration_ms": 0,
        }

        # Process each language
        for language in languages:
            await self._process_language(repo, language, include_tests, stats)

        # Count totals
        self._count_totals(stats)

        stats["duration_ms"] = int((time.time() - start_time) * 1000)

        return stats

    async def _process_language(
        self, repo: Path, language: str, include_tests: bool, stats: dict[str, int]
    ) -> None:
        """Process files for a specific programming language."""
        file_pattern = self._get_file_pattern(language)
        if not file_pattern:
            return

        # Find all source files
        for file_path in repo.rglob(file_pattern):
            if not include_tests and self._should_skip_test_file(file_path):
                continue

            if self._should_skip_non_source_dir(file_path):
                continue

            # Analyze the file
            if language == "python":
                await self._analyze_python_file(file_path)

            stats["files_indexed"] += 1

    def _get_file_pattern(self, language: str) -> str | None:
        """Get file pattern for a specific language."""
        if language == "python":
            return "*.py"
        if language in ("javascript", "typescript"):
            return "*.js" if language == "javascript" else "*.ts"
        return None

    def _should_skip_test_file(self, file_path: Path) -> bool:
        """Check if a test file should be skipped."""
        # Skip if "test" is in directory path
        if "test" in file_path.parts:
            return True
        # Skip if filename starts with "test_"
        if file_path.name.startswith("test_"):
            return True
        # Skip if filename is "test_*.py"
        if file_path.stem.startswith("test_"):
            return True
        return False

    def _should_skip_non_source_dir(self, file_path: Path) -> bool:
        """Check if file is in a non-source directory that should be skipped."""
        non_source_dirs = {"__pycache__", ".venv", "venv", "node_modules"}
        return any(part in file_path.parts for part in non_source_dirs)

    def _count_totals(self, stats: dict[str, int]) -> None:
        """Count total functions, classes, and imports."""
        for node in self.nodes.values():
            if node.node_type == "function":
                stats["functions_indexed"] += 1
            elif node.node_type == "class":
                stats["classes_indexed"] += 1
            elif node.node_type == "import":
                stats["imports_indexed"] += 1

        # Count calls
        for callees in self._call_graph.values():
            stats["calls_indexed"] += len(callees)

    async def _analyze_python_file(self, file_path: Path) -> None:
        """Analyze a Python source file and extract code structure.

        Args:
            file_path: Path to the Python file
        """
        source = await self._read_file_content(file_path)
        if source is None:
            return

        tree = await self._parse_ast(source, file_path)
        if tree is None:
            return

        # Create file node
        file_id = str(file_path.relative_to(self.project_path))
        file_node = self._create_file_node(file_path, file_id)
        self.nodes[file_id] = file_node
        self._file_nodes[file_id] = file_node

        # Process imports and definitions
        await self._process_file_content(tree, file_node, file_id)

    async def _read_file_content(self, file_path: Path) -> str | None:
        """Read file content, returning None if there's an error."""
        try:
            return file_path.read_text(encoding="utf-8")
        except (OSError, UnicodeDecodeError):
            return None

    async def _parse_ast(self, source: str, file_path: Path) -> ast.AST | None:
        """Parse AST from source, returning None if there's a syntax error."""
        try:
            return ast.parse(source, filename=str(file_path))
        except SyntaxError:
            return None

    def _create_file_node(self, file_path: Path, file_id: str) -> FileNode:
        """Create a file node for the given file."""
        return FileNode(
            id=file_id,
            name=file_path.name,
            file_id=file_id,
            node_type="file",
            path=str(file_path),
            language="python",
        )

    async def _process_file_content(
        self, tree: ast.AST, file_node: FileNode, file_id: str
    ) -> None:
        """Process the AST tree to extract imports, classes, and functions."""
        self._extract_and_add_imports(tree, file_node, file_id)
        self._extract_and_add_definitions(tree, file_node, file_id)

    def _extract_and_add_imports(
        self, tree: ast.AST, file_node: FileNode, file_id: str
    ) -> None:
        """Extract imports from the AST and add them to the file node."""
        for node in ast.walk(tree):
            if isinstance(node, (ast.Import, ast.ImportFrom)):
                import_node = self._extract_import(node, file_id)
                if import_node:
                    self.nodes[import_node.id] = import_node
                    file_node.imports.append(import_node.id)

    def _extract_and_add_definitions(
        self, tree: ast.AST, file_node: FileNode, file_id: str
    ) -> None:
        """Extract classes and functions from the AST and add them to the file node."""
        for node in ast.walk(tree):
            if isinstance(node, ast.ClassDef):
                class_node = self._extract_class(node, file_id, Path(file_node.path))
                if class_node:
                    self.nodes[class_node.id] = class_node
                    file_node.classes.append(class_node.id)

            elif isinstance(node, ast.FunctionDef):
                # Skip methods (they're part of classes)
                if not self._is_method(node, tree):
                    func_node = self._extract_function(
                        node, file_id, Path(file_node.path)
                    )
                    if func_node:
                        self.nodes[func_node.id] = func_node
                        file_node.functions.append(func_node.id)

    def _is_method(self, func_node: ast.FunctionDef, tree: ast.AST) -> bool:
        """Check if a function is a method (defined inside a class)."""
        for parent in ast.walk(tree):
            if isinstance(parent, ast.ClassDef):
                for child in parent.body:
                    if child is func_node:
                        return True
        return False

    def _extract_import(
        self, node: ast.Import | ast.ImportFrom, file_id: str
    ) -> ImportNode | None:
        """Extract import information from an AST import node."""
        if isinstance(node, ast.Import):
            for alias in node.names:
                import_id = f"{file_id}:import:{alias.name}"
                return ImportNode(
                    id=import_id,
                    name=alias.name,
                    file_id=file_id,
                    node_type="import",
                    module=alias.name,
                    alias=alias.asname,
                    is_from_import=False,
                )
        if isinstance(node, ast.ImportFrom):
            module = node.module or ""
            for alias in node.names:
                import_id = f"{file_id}:from:{module}:{alias.name}"
                return ImportNode(
                    id=import_id,
                    name=alias.name,
                    file_id=file_id,
                    node_type="import",
                    module=module,
                    alias=alias.asname,
                    is_from_import=True,
                    imported_names=[alias.name],
                )
        return None

    def _extract_class(
        self, node: ast.ClassDef, file_id: str, file_path: Path
    ) -> ClassNode | None:
        """Extract class information from an AST ClassDef node."""
        class_id = f"{file_id}:class:{node.name}"

        # Get base classes
        base_classes = []
        for base in node.bases:
            if isinstance(base, ast.Name):
                base_classes.append(base.id)
            elif isinstance(base, ast.Attribute):
                base_classes.append(ast.unparse(base))

        # Get docstring
        docstring = ast.get_docstring(node)

        # Calculate span
        start_line = node.lineno
        end_line = (
            node.end_lineno
            if hasattr(node, "end_lineno") and node.end_lineno is not None
            else start_line
        )

        return ClassNode(
            id=class_id,
            name=node.name,
            file_id=file_id,
            node_type="class",
            start_line=start_line,
            end_line=end_line,
            base_classes=base_classes,
            docstring=docstring,
        )

    def _extract_function(
        self, node: ast.FunctionDef, file_id: str, file_path: Path
    ) -> FunctionNode | None:
        """Extract function information from an AST FunctionDef node."""
        func_id = f"{file_id}:function:{node.name}"

        # Check if it's exported (not starting with _)
        is_export = not node.name.startswith("_")

        # Get docstring
        docstring = ast.get_docstring(node)

        # Calculate complexity (simplified - count branches)
        complexity = 1
        for child in ast.walk(node):
            if isinstance(child, (ast.If, ast.While, ast.For, ast.Try)):
                complexity += 1

        # Extract function calls
        calls = []
        for child in ast.walk(node):
            if isinstance(child, ast.Call):
                call_name = self._extract_call_name(child)
                if call_name:
                    calls.append(call_name)
                    # Add to call graph
                    self._call_graph[func_id].add(call_name)

        # Calculate span
        start_line = node.lineno
        end_line = (
            node.end_lineno
            if hasattr(node, "end_lineno") and node.end_lineno is not None
            else start_line
        )

        return FunctionNode(
            id=func_id,
            name=node.name,
            file_id=file_id,
            node_type="function",
            is_export=is_export,
            start_line=start_line,
            end_line=end_line,
            calls=calls,
            docstring=docstring,
            complexity=complexity,
        )

    def _extract_call_name(self, call_node: ast.Call) -> str | None:
        """Extract the name of a function call."""
        if isinstance(call_node.func, ast.Name):
            return call_node.func.id
        elif isinstance(call_node.func, ast.Attribute):
            return ast.unparse(call_node.func)
        return None

    async def get_function_context(
        self,
        function_name: str,
        include_callers: bool = True,
        include_callees: bool = True,
        max_depth: int = 3,
    ) -> dict[str, Any]:
        """
        Get comprehensive context for a function.

        Args:
            function_name: Name of the function to query
            include_callers: Include functions that call this function
            include_callees: Include functions that this function calls
            max_depth: Maximum depth for call graph traversal

        Returns:
            Function context including callers, callees, and metadata
        """
        func_node = self._find_function_node(function_name)
        if not func_node:
            raise ValueError(f"Function not found: {function_name}")

        context: dict[str, Any] = {
            "function": self._build_function_metadata(func_node),
            "callers": [],
            "callees": [],
        }

        # Get callers and callees based on flags
        if include_callers:
            context["callers"] = self._get_callers(function_name)

        if include_callees:
            context["callees"] = self._get_callees(func_node)

        return context

    def _find_function_node(self, function_name: str) -> FunctionNode | None:
        """Find a function node by name."""
        for node in self.nodes.values():
            if node.node_type == "function" and node.name == function_name:
                if isinstance(node, FunctionNode):
                    return node
        return None

    def _build_function_metadata(self, func_node: FunctionNode) -> dict[str, Any]:
        """Build function metadata dictionary."""
        return {
            "name": func_node.name,
            "file": func_node.file_id,
            "start_line": func_node.start_line,
            "end_line": func_node.end_line,
            "is_export": func_node.is_export,
        }

    def _get_callers(self, function_name: str) -> list[dict[str, Any]]:
        """Get list of functions that call the specified function."""
        callers = []
        for caller_id, callees in self._call_graph.items():
            if function_name in callees:
                caller_node = self.nodes.get(caller_id)
                if caller_node and isinstance(caller_node, FunctionNode):
                    callers.append(
                        {
                            "name": caller_node.name,
                            "file": caller_node.file_id,
                            "line": caller_node.start_line,
                        }
                    )
        return callers

    def _get_callees(self, func_node: FunctionNode) -> list[dict[str, Any]]:
        """Get list of functions called by the specified function."""
        callees = []
        for callee_name in func_node.calls:
            callee_node = self._find_function_by_name(callee_name)
            if callee_node:
                callees.append(
                    {
                        "name": callee_node.name,
                        "file": callee_node.file_id,
                        "line": callee_node.start_line,
                    }
                )
        return callees

    def _find_function_by_name(self, name: str) -> FunctionNode | None:
        """Find a function node by name."""
        for node in self.nodes.values():
            if (
                node.node_type == "function"
                and node.name == name
                and isinstance(node, FunctionNode)
            ):
                return node
        return None

    async def find_related_files(
        self,
        file_path: str,
        relationship_type: str = "all",
        limit: int = 20,
    ) -> list[dict[str, Any]]:
        """
        Find files related to a specific file based on imports and call relationships.

        Args:
            file_path: Path to source file (relative to project_path)
            relationship_type: Type of relationship (imports, imported_by, calls, called_by, all)
            limit: Maximum number of related files to return

        Returns:
            List of related files with relationship types and strength
        """
        file_id = str(Path(file_path))
        related_files: list[dict[str, Any]] = []

        # Get imports from this file
        imports, imports_from = self._get_file_imports(file_id)

        # Add relationships based on type
        if relationship_type in ("imported_by", "all"):
            related_files.extend(self._find_imported_by_relationships(file_id))

        if relationship_type in ("imports", "all"):
            related_files.extend(
                self._find_imports_relationships(imports | imports_from)
            )

        if relationship_type in ("calls", "all"):
            file_func_ids = self._get_file_function_ids(file_id)
            related_files.extend(self._find_calls_relationships(file_func_ids, file_id))

        if relationship_type in ("called_by", "all"):
            file_func_ids = self._get_file_function_ids(file_id)
            related_files.extend(self._find_called_by_relationships(file_func_ids))

        # Aggregate by file and sort by strength
        result = self._aggregate_and_sort_results(related_files)

        return result[:limit]

    def _get_file_imports(self, file_id: str) -> tuple[set[str], set[str]]:
        """Get imports from a specific file."""
        imports = set()
        imports_from = set()
        for node in self.nodes.values():
            if node.file_id == file_id and node.node_type == "import":
                if isinstance(node, ImportNode):
                    if node.is_from_import:
                        imports_from.add(node.module)
                    else:
                        imports.add(node.module)
        return imports, imports_from

    def _find_imported_by_relationships(self, file_id: str) -> list[dict[str, Any]]:
        """Find files that import this file's modules."""
        related_files = []
        for node in self.nodes.values():
            if node.node_type == "import" and node.file_id != file_id:
                if isinstance(node, ImportNode):
                    # Check if this import references our file
                    if any(
                        imp in file_id for imp in [node.module] + node.imported_names
                    ):
                        related_files.append(
                            {
                                "file_path": node.file_id,
                                "relationship": "imported_by",
                                "strength": 1,
                            }
                        )
        return related_files

    def _find_imports_relationships(
        self, all_imports: set[str]
    ) -> list[dict[str, Any]]:
        """Find files that this file imports."""
        related_files = []
        for module in all_imports:
            for node in self.nodes.values():
                if node.node_type == "file" and module in node.file_id:
                    related_files.append(
                        {
                            "file_path": node.file_id,
                            "relationship": "imports",
                            "strength": 1,
                        }
                    )
        return related_files

    def _get_file_function_ids(self, file_id: str) -> list[str]:
        """Get function IDs for a specific file."""
        return [
            node.id
            for node in self.nodes.values()
            if node.file_id == file_id and node.node_type == "function"
        ]

    def _find_calls_relationships(
        self, file_func_ids: list[str], file_id: str
    ) -> list[dict[str, Any]]:
        """Find files with calls relationships."""
        related_files = []
        for func_id in file_func_ids:
            for callee in self._call_graph.get(func_id, set()):
                callee_node = self.nodes.get(f"{file_id}:function:{callee}")
                if callee_node:
                    related_files.append(
                        {
                            "file_path": callee_node.file_id,
                            "relationship": "calls",
                            "strength": 1,
                        }
                    )
        return related_files

    def _find_called_by_relationships(
        self, file_func_ids: list[str]
    ) -> list[dict[str, Any]]:
        """Find files that call functions in this file."""
        related_files = []
        for caller_id, callees in self._call_graph.items():
            if any(func_id in callees for func_id in file_func_ids):
                caller_node = self.nodes.get(caller_id)
                if caller_node:
                    related_files.append(
                        {
                            "file_path": caller_node.file_id,
                            "relationship": "called_by",
                            "strength": 1,
                        }
                    )
        return related_files

    def _aggregate_and_sort_results(
        self, related_files: list[dict[str, Any]]
    ) -> list[dict[str, Any]]:
        """Aggregate by file and sort by strength."""
        aggregated: dict[str, dict[str, Any]] = {}
        for rf in related_files:
            fp = rf["file_path"]
            if fp not in aggregated:
                aggregated[fp] = {"file_path": fp, "relationship": [], "strength": 0}
            aggregated[fp]["relationship"].append(rf["relationship"])
            aggregated[fp]["strength"] += rf["strength"]

        # Convert to list and sort
        result = list(aggregated.values())
        result.sort(key=operator.itemgetter("strength"), reverse=True)

        return result



"""Minimal MCP server example - tools only.

This example demonstrates the MinimalServer profile:
- Basic tool registration
- Simple configuration
- Fast startup
- Minimal dependencies

Best for:
    - Quick prototypes
    - Simple utility servers
    - Stateless tools
    - Development environments

Usage:
    python examples/minimal_server.py
"""

from __future__ import annotations

from mcp_common.profiles import MinimalServer
from mcp_common.ui import ServerPanels


def create_minimal_server() -> MinimalServer:
    """Create a minimal MCP server with basic tools.

    Returns:
        MinimalServer instance with registered tools

    Example:
        >>> server = create_minimal_server()
        >>> server.run()
    """
    # Create minimal server
    server = MinimalServer(name="minimal-server")

    # Register tools using decorators
    @server.tool()
    def hello(name: str) -> str:
        """Say hello to someone.

        Args:
            name: Person's name

        Returns:
            Greeting message
        """
        return f"Hello, {name}!"

    @server.tool()
    def add(a: int, b: int) -> int:
        """Add two numbers.

        Args:
            a: First number
            b: Second number

        Returns:
            Sum of the numbers
        """
        return a + b

    @server.tool()
    def multiply(a: int, b: int) -> int:
        """Multiply two numbers.

        Args:
            a: First number
            b: Second number

        Returns:
            Product of the numbers
        """
        return a * b

    @server.tool()
    def reverse_string(text: str) -> str:
        """Reverse a string.

        Args:
            text: Text to reverse

        Returns:
            Reversed text
        """
        return text[::-1]

    return server


def main() -> None:
    """Main entry point for minimal server example."""
    # Create server
    server = create_minimal_server()

    # Display startup panel
    ServerPanels.startup_success(
        server_name="Minimal MCP Server",
        version="1.0.0",
        features=[
            "Basic Tools",
            "Simple Configuration",
            "Fast Startup",
            "Minimal Dependencies",
        ],
    )

    # Show available tools
    tools = server.list_tools()
    print(f"\nAvailable tools: {', '.join(tools)}")

    # Show health check
    health = server.health_check()
    print(f"\nHealth status: {health['status']}")
    print(f"Server: {health['server']}")
    print(f"Tools registered: {health['tools']}")

    # In a real implementation, you would call server.run() here
    # which would start the actual MCP server (e.g., FastMCP)
    print("\nNote: This is a profile demonstration.")
    print("In production, you would integrate with FastMCP or native MCP server.")
    print("\nExample integration:")
    print("    from fastmcp import FastMCP")
    print("    mcp = FastMCP('minimal-server')")
    print("    for name, tool in server._tools.items():")
    print("        mcp.add_tool(tool)")
    print("    mcp.run()")


if __name__ == "__main__":
    main()



"""Full MCP server example - production-ready with all features.

This example demonstrates the FullServer profile:
- Tool registration
- Resource management
- Prompt templates
- Authentication support
- Telemetry support
- Multi-worker support

Best for:
    - Production servers
    - Multi-user environments
    - Servers requiring authentication
    - Servers needing observability
    - High-traffic deployments

Usage:
    python examples/full_server.py
"""

from __future__ import annotations

import json
from typing import Any

from mcp_common.profiles import FullServer
from mcp_common.ui import ServerPanels


# Mock auth and telemetry backends (in production, import from actual modules)
class MockAuthBackend:
    """Mock authentication backend for demonstration."""

    def __init__(self, secret: str) -> None:
        self.secret = secret
        self.enabled = True

    def authenticate(self, token: str) -> bool:
        """Authenticate a token."""
        return token == self.secret


class MockTelemetryBackend:
    """Mock telemetry backend for demonstration."""

    def __init__(self, endpoint: str) -> None:
        self.endpoint = endpoint
        self.enabled = True

    def trace(self, operation: str) -> None:
        """Trace an operation."""
        pass


def create_full_server() -> FullServer:
    """Create a full-featured MCP server with all components.

    Returns:
        FullServer instance with tools, resources, and prompts

    Example:
        >>> auth = MockAuthBackend(secret="my-secret")
        >>> telemetry = MockTelemetryBackend(endpoint="http://jaeger:4317")
        >>> server = create_full_server()
        >>> server.run(workers=4)
    """
    # Create mock auth and telemetry backends
    auth = MockAuthBackend(secret="demo-secret")
    telemetry = MockTelemetryBackend(endpoint="http://jaeger:4317")

    # Create full server
    server = FullServer(
        name="full-server",
        description="Production-ready MCP Server with all features",
        auth=auth,
        telemetry=telemetry,
    )

    # Register tools using decorators
    @server.tool()
    def search(query: str, filters: dict[str, Any] | None = None) -> dict[str, Any]:
        """Search for items with optional filters.

        Args:
            query: Search query string
            filters: Optional filter criteria

        Returns:
            Dictionary with search results
        """
        # Mock search with authentication and telemetry hooks
        if server.auth:
            # In production: validate token
            pass

        if server.telemetry:
            # In production: trace this operation
            pass

        results = [
            {"id": 1, "title": f"Result 1 for '{query}'"},
            {"id": 2, "title": f"Result 2 for '{query}'"},
        ]

        if filters:
            results = [r for r in results if all(r.get(k) == v for k, v in filters.items())]

        return {"query": query, "results": results, "count": len(results), "filters": filters}

    @server.tool()
    def process_data(data: dict[str, Any], operations: list[str]) -> dict[str, Any]:
        """Process data with multiple operations.

        Args:
            data: Input data dictionary
            operations: List of operations to apply

        Returns:
            Processed data dictionary
        """
        result = data.copy()

        for op in operations:
            if op == "uppercase":
                result = {k: str(v).upper() if isinstance(v, str) else v for k, v in result.items()}
            elif op == "reverse":
                result = {k: str(v)[::-1] if isinstance(v, str) else v for k, v in result.items()}
            elif op == "count":
                result["_count"] = len(result)

        return {"original": data, "operations": operations, "result": result}

    # Register resource handlers
    @server.resource("config://{env}/{name}")
    def get_config(env: str, name: str) -> str:
        """Get configuration by environment and name.

        Args:
            env: Environment (dev, staging, prod)
            name: Configuration name

        Returns:
            Configuration as JSON string
        """
        # Mock environment-specific configs
        configs = {
            "dev": {
                "database": {"host": "localhost", "port": 5432, "name": "mydb_dev"},
                "api": {"endpoint": "http://localhost:8000", "timeout": 30},
            },
            "prod": {
                "database": {"host": "db.prod.example.com", "port": 5432, "name": "mydb_prod"},
                "api": {"endpoint": "https://api.example.com", "timeout": 10},
            },
        }
        return json.dumps(configs.get(env, {}).get(name, {}))

    @server.resource("data://{table}/{id}")
    def get_record(table: str, id: str) -> str:
        """Get a specific record from a table.

        Args:
            table: Table name
            id: Record ID

        Returns:
            Record data as JSON string
        """
        # Mock record data
        records = {
            "users": {
                "1": {"id": 1, "name": "Alice", "email": "alice@example.com"},
                "2": {"id": 2, "name": "Bob", "email": "bob@example.com"},
            },
            "products": {
                "1": {"id": 1, "name": "Product A", "price": 29.99},
                "2": {"id": 2, "name": "Product B", "price": 49.99},
            },
        }
        return json.dumps(records.get(table, {}).get(id, {}))

    # Register prompt templates
    @server.prompt("analyze")
    def analyze_prompt(data: str) -> str:
        """Generate a prompt for data analysis.

        Args:
            data: Data to analyze

        Returns:
            Analysis prompt string
        """
        return f"""Please analyze the following data:

{data}

Provide insights on:
1. Trends and patterns
2. Outliers and anomalies
3. Key takeaways
4. Recommendations
"""

    @server.prompt("summarize")
    def summarize_prompt(content: str, max_length: int = 200) -> str:
        """Generate a prompt for summarization.

        Args:
            content: Content to summarize
            max_length: Maximum summary length

        Returns:
            Summarization prompt string
        """
        return f"""Please summarize the following content in {max_length} words or less:

{content}

Focus on:
- Main points
- Key insights
- Action items
"""

    @server.prompt("code_review")
    def code_review_prompt(code: str, language: str = "Python") -> str:
        """Generate a prompt for code review.

        Args:
            code: Code to review
            language: Programming language

        Returns:
            Code review prompt string
        """
        return f"""Please review the following {language} code:

```{language.lower()}
{code}
```

Provide feedback on:
1. Code quality and readability
2. Potential bugs or issues
3. Performance considerations
4. Security concerns
5. Best practices
6. Suggestions for improvement
"""

    return server


def main() -> None:
    """Main entry point for full server example."""
    # Create server
    server = create_full_server()

    # Display startup panel
    ServerPanels.startup_success(
        server_name="Full MCP Server",
        version="1.0.0",
        features=[
            "Tools (search, process)",
            "Resources (config, data)",
            "Prompts (analyze, summarize, code_review)",
            "Authentication",
            "Telemetry",
            "Multi-worker support",
        ],
    )

    # Show all available components
    tools = server.list_tools()
    resources = server.list_resources()
    prompts = server.list_prompts()

    print(f"\nAvailable tools: {', '.join(tools)}")
    print(f"Available resources: {', '.join(resources)}")
    print(f"Available prompts: {', '.join(prompts)}")

    # Show comprehensive health check
    health = server.health_check()
    print(f"\nHealth status: {health['status']}")
    print(f"Server: {health['server']}")
    print(f"Description: {health['description']}")
    print(f"Tools: {health['tools']}")
    print(f"Resources: {health['resources']}")
    print(f"Prompts: {health['prompts']}")
    print(f"Auth enabled: {health['auth']['enabled']}")
    print(f"Telemetry enabled: {health['telemetry']['enabled']}")
    print(f"Workers: {health['workers']}")

    # Demonstrate prompt templates
    print("\n--- Prompt Template Examples ---")
    analyze_prompt = server.get_prompt("analyze")
    if analyze_prompt:
        prompt = analyze_prompt(data="Sample data for analysis")
        print(f"\nAnalyze prompt preview:\n{prompt[:200]}...")

    # In a real implementation, you would call server.run() here
    print("\n" + "=" * 80)
    print("Note: This is a profile demonstration.")
    print("In production, you would integrate with FastMCP or native MCP server.")
    print("\nExample integration:")
    print("    from fastmcp import FastMCP")
    print("    mcp = FastMCP('full-server')")
    print("    for name, tool in server._tools.items():")
    print("        mcp.add_tool(tool)")
    print("    for uri, handler in server._resources.items():")
    print("        mcp.add_resource(uri, handler)")
    print("    for name, template in server._prompts.items():")
    print("        mcp.add_prompt(name, template)")
    print("    mcp.run()")
    print("\nWith auth and telemetry:")
    print("    # Auth middleware would check tokens")
    print("    # Telemetry middleware would trace operations")
    print("    server.run(workers=4)")


if __name__ == "__main__":
    main()



"""WebSocket server base class for MCP services.

Provides a unified WebSocket server implementation that all ecosystem
services can extend for real-time communication.
"""

from __future__ import annotations

import logging
import ssl
import time
import uuid
from abc import ABC, abstractmethod
from logging import Logger
from typing import Any, Callable

try:
    import websockets
    from websockets.server import WebSocketServerProtocol
    WEBSOCKETS_AVAILABLE = True
except ImportError:
    WEBSOCKETS_AVAILABLE = False

from .protocol import (
    WebSocketMessage,
    WebSocketProtocol,
    MessageType,
    EventTypes,
)
from .tls import (
    create_ssl_context,
    create_development_ssl_context,
    get_tls_config_from_env,
    validate_certificate,
)
from .metrics import WebSocketMetrics

logger: Logger = logging.getLogger(__name__)


class WebSocketServer(ABC):
    """
    Base WebSocket server for MCP services.

    Features:
    - Room-based broadcasting
    - Request/response correlation
    - Automatic reconnection handling
    - JWT authentication integration
    - TLS/WSS support for secure connections
    - Message rate limiting
    - Prometheus metrics
    - Graceful shutdown

    Usage:
        class MyServiceWebSocket(WebSocketServer):
            async def on_connect(self, websocket, session_id):
                await self.join_room(f"session:{session_id}")

            async def on_message(self, websocket, message):
                if message.event == "subscribe":
                    await self.subscribe_to_session(websocket, message.data["session_id"])

    Implementation:
        server = MyServiceWebSocket(host="127.0.0.1", port=8688)
        await server.start()

    With TLS:
        server = MyServiceWebSocket(
            host="0.0.0.0",
            port=8688,
            cert_file="/path/to/cert.pem",
            key_file="/path/to/key.pem"
        )
        await server.start()

    With auto-generated development certificate:
        server = MyServiceWebSocket(host="127.0.0.1", port=8688, tls_enabled=True)
        await server.start()

    With Prometheus metrics:
        server = MyServiceWebSocket(
            host="127.0.0.1",
            port=8688,
            enable_metrics=True,
            metrics_port=9090
        )
        await server.start()
    """

    def __init__(
        self,
        host: str = "127.0.0.1",
        port: int = 8688,
        max_connections: int = 1000,
        message_rate_limit: int = 100,  # messages per second per connection
        authenticator: Any | None = None,  # WebSocketAuthenticator instance
        require_auth: bool = False,  # Require JWT authentication
        ssl_context: ssl.SSLContext | None = None,
        cert_file: str | None = None,
        key_file: str | None = None,
        ca_file: str | None = None,
        tls_enabled: bool = False,
        verify_client: bool = False,
        auto_cert: bool = False,
        server_name: str | None = None,  # For metrics
        enable_metrics: bool = False,  # Enable Prometheus metrics
        metrics_port: int = 9090,  # Prometheus metrics port
    ):
        """Initialize WebSocket server.

        Args:
            host: Server host address
            port: Server port number
            max_connections: Maximum concurrent connections
            message_rate_limit: Messages per second per connection
            authenticator: WebSocketAuthenticator instance for JWT auth
            require_auth: Whether to require authentication for connections
            ssl_context: Pre-configured SSL context (overrides cert/key files)
            cert_file: Path to TLS certificate file (PEM format)
            key_file: Path to TLS private key file (PEM format)
            ca_file: Path to CA file for client verification
            tls_enabled: Enable TLS (generates self-signed cert if no cert provided)
            verify_client: Verify client certificates
            auto_cert: Auto-generate self-signed certificate for development
            server_name: Server name for metrics (defaults to class name)
            enable_metrics: Enable Prometheus metrics collection
            metrics_port: Port for Prometheus metrics server
        """
        if not WEBSOCKETS_AVAILABLE:
            raise ImportError(
                "websockets package is required. "
                "Install it with: pip install websockets"
            )

        self.host = host
        self.port = port
        self.max_connections = max_connections
        self.message_rate_limit = message_rate_limit
        self.authenticator = authenticator
        self.require_auth = require_auth

        # TLS configuration
        self.ssl_context = ssl_context
        self.cert_file = cert_file
        self.key_file = key_file
        self.ca_file = ca_file
        self.tls_enabled = tls_enabled
        self.verify_client = verify_client
        self.auto_cert = auto_cert

        # Auto-generated certificate paths (for cleanup)
        self._auto_cert_path: str | None = None
        self._auto_key_path: str | None = None

        # Connection management
        self.connections: dict[str, Any] = {}  # connection_id -> websocket
        self.connection_rooms: dict[str, set[str]] = {}  # room_id -> {connection_ids}
        self.room_connections: dict[str, str] = {}  # connection_id -> room_id

        # Event handlers
        self.event_handlers: dict[str, set[Callable]] = {}

        # Server state
        self.server: Any | None = None
        self.is_running = False

        # Metrics
        self.enable_metrics = enable_metrics
        self.metrics_port = metrics_port
        self.server_name = server_name or self.__class__.__name__
        self.metrics: WebSocketMetrics | None = None

        # Initialize SSL context if TLS enabled
        if self.tls_enabled and self.ssl_context is None:
            self._initialize_ssl_context()

        # Initialize metrics
        if self.enable_metrics:
            self.metrics = WebSocketMetrics(
                server_name=self.server_name,
                tls_enabled=bool(self.ssl_context),
                enabled=True
            )

    def _initialize_ssl_context(self) -> None:
        """Initialize SSL context from certificates or auto-generate.

        Raises:
            FileNotFoundError: If cert/key files specified but not found
            RuntimeError: If SSL context creation fails
        """
        try:
            # Use provided cert/key files
            if self.cert_file and self.key_file:
                self.ssl_context = create_ssl_context(
                    cert_file=self.cert_file,
                    key_file=self.key_file,
                    ca_file=self.ca_file,
                    verify_client=self.verify_client,
                )
                logger.info(f"Loaded TLS certificate: {self.cert_file}")

                # Validate certificate
                validation = validate_certificate(self.cert_file)
                if not validation["valid"]:
                    logger.warning(
                        f"Certificate validation warning: {validation['error']}"
                    )
                if validation.get("days_remaining"):
                    logger.info(
                        f"Certificate valid for {validation['days_remaining']} days"
                    )

            # Auto-generate self-signed certificate
            elif self.auto_cert or not (self.cert_file and self.key_file):
                self.ssl_context, self._auto_cert_path, self._auto_key_path = (
                    create_development_ssl_context(
                        common_name=self.host,
                        dns_names=[self.host, "localhost"],
                    )
                )
                logger.warning(
                    f"Using auto-generated self-signed certificate for development. "
                    f"Cert: {self._auto_cert_path}"
                )

        except Exception as e:
            raise RuntimeError(f"Failed to initialize SSL context: {e}") from e

    @property
    def uri(self) -> str:
        """Get the WebSocket server URI.

        Returns:
            WebSocket URI (ws:// or wss://)
        """
        scheme = "wss" if self.ssl_context else "ws"
        return f"{scheme}://{self.host}:{self.port}"

    @abstractmethod
    async def on_connect(self, websocket: Any, connection_id: str):
        """
        Handle new WebSocket connection.

        Args:
            websocket: WebSocket connection object
            connection_id: Unique connection identifier
        """
        pass

    @abstractmethod
    async def on_disconnect(self, websocket: Any, connection_id: str):
        """
        Handle WebSocket disconnection.

        Args:
            websocket: WebSocket connection object
            connection_id: Unique connection identifier
        """
        pass

    @abstractmethod
    async def on_message(self, websocket: Any, message: WebSocketMessage):
        """
        Handle incoming WebSocket message.

        Args:
            websocket: WebSocket connection object
            message: Decoded message
        """
        pass

    def authenticate_websocket(self, token: str) -> dict[str, Any] | None:
        """Authenticate WebSocket connection using JWT.

        Args:
            token: JWT token from connection

        Returns:
            User payload if authenticated, None otherwise

        Example:
            >>> # In on_connect handler
            >>> token = await websocket.recv()
            >>> user = self.authenticate_websocket(token)
            >>> if user is None:
            >>>     await websocket.close(code=1008, reason="Invalid token")
        """
        if self.authenticator is None:
            logger.warning("Authentication attempted but no authenticator configured")
            return None

        return self.authenticator.authenticate_connection(token)

    async def start(self):
        """Start the WebSocket server."""
        if self.is_running:
            logger.warning(f"WebSocket server already running on {self.uri}")
            return

        logger.info(f"Starting WebSocket server on {self.uri}")

        # Start metrics server if enabled
        if self.enable_metrics and self.metrics:
            self.metrics.start_metrics_server(self.metrics_port)

        async def handler(websocket):
            # Check connection limit
            if len(self.connections) >= self.max_connections:
                await websocket.close(1013, "Server at maximum capacity")
                if self.metrics:
                    self.metrics.on_connection_error("max_connections")
                return

            connection_id = str(uuid.uuid4())

            # Handle authentication if required
            if self.require_auth and self.authenticator:
                try:
                    # Receive first message which should contain auth token
                    auth_message = await websocket.recv()
                    auth_data = WebSocketProtocol.decode(auth_message)

                    if auth_data.type == MessageType.REQUEST and auth_data.event == "auth":
                        token = auth_data.data.get("token")
                        user = self.authenticate_websocket(token)

                        if user is None:
                            # Send auth error and close
                            error = WebSocketProtocol.create_error(
                                error_code="AUTH_FAILED",
                                error_message="Invalid or expired token",
                                correlation_id=auth_data.correlation_id,
                            )
                            await websocket.send(WebSocketProtocol.encode(error))
                            await websocket.close(1008, "Authentication failed")
                            if self.metrics:
                                self.metrics.on_connection_error("auth_failed")
                            return

                        # Store user on websocket object
                        websocket.user = user
                        logger.info(
                            f"Authenticated connection {connection_id} "
                            f"for user {user.get('user_id')}"
                        )

                        # Send auth success
                        response = WebSocketProtocol.create_response(
                            auth_data,
                            200,
                            {"status": "authenticated", "user_id": user.get("user_id")}
                        )
                        await websocket.send(WebSocketProtocol.encode(response))
                    else:
                        # First message must be auth
                        error = WebSocketProtocol.create_error(
                            error_code="AUTH_REQUIRED",
                            error_message="Authentication required",
                        )
                        await websocket.send(WebSocketProtocol.encode(error))
                        await websocket.close(1008, "Authentication required")
                        if self.metrics:
                            self.metrics.on_connection_error("auth_required")
                        return

                except Exception as e:
                    logger.error(f"Authentication error: {e}")
                    await websocket.close(1011, "Authentication error")
                    if self.metrics:
                        self.metrics.on_connection_error("auth_error")
                    return

            try:
                # Record connection metrics
                if self.metrics:
                    self.metrics.on_connect(connection_id)

                await self.on_connect(websocket, connection_id)
                self.connections[connection_id] = websocket

                # Message loop
                async for message in websocket:
                    # Rate limiting check could go here
                    start_time = time.time()
                    try:
                        decoded = WebSocketProtocol.decode(message)

                        # Record received message metric
                        if self.metrics:
                            self.metrics.on_message_received(str(decoded.type))

                        await self.on_message(websocket, decoded)

                        # Record latency
                        if self.metrics:
                            self.metrics.observe_latency(
                                str(decoded.type),
                                time.time() - start_time
                            )
                    except Exception as e:
                        logger.error(f"Error decoding message: {e}")
                        error_msg = WebSocketProtocol.create_error(
                            error_code="DECODE_ERROR",
                            error_message=str(e)
                        )
                        await websocket.send(WebSocketProtocol.encode(error_msg))

                        if self.metrics:
                            self.metrics.on_message_error("decode_error")

            except websockets.exceptions.ConnectionClosed:
                logger.info(f"Connection {connection_id} closed")
            finally:
                await self.on_disconnect(websocket, connection_id)
                if connection_id in self.connections:
                    del self.connections[connection_id]

                # Record disconnection metrics
                if self.metrics:
                    self.metrics.on_disconnect(connection_id)

        # Start server with or without SSL
        self.server = await websockets.serve(
            handler,
            self.host,
            self.port,
            ssl=self.ssl_context,
        )
        self.is_running = True

        tls_mode = "WSS" if self.ssl_context else "WS"
        logger.info(
            f"WebSocket server listening on {self.uri} "
            f"({tls_mode}, {len(self.connections)} connections)"
        )

    async def stop(self):
        """Stop the WebSocket server."""
        if not self.is_running:
            return

        logger.info("Stopping WebSocket server")

        # Close all connections
        for connection_id, websocket in list(self.connections.items()):
            try:
                await websocket.close()
            except Exception:
                pass

        self.is_running = False

        # Clean up auto-generated certificate files
        self._cleanup_auto_cert()

    def _cleanup_auto_cert(self) -> None:
        """Clean up auto-generated certificate files."""
        import os

        if self._auto_cert_path and os.path.exists(self._auto_cert_path):
            try:
                os.unlink(self._auto_cert_path)
                logger.debug(f"Cleaned up auto-generated cert: {self._auto_cert_path}")
            except Exception as e:
                logger.warning(f"Failed to cleanup cert file: {e}")

        if self._auto_key_path and os.path.exists(self._auto_key_path):
            try:
                os.unlink(self._auto_key_path)
                logger.debug(f"Cleaned up auto-generated key: {self._auto_key_path}")
            except Exception as e:
                logger.warning(f"Failed to cleanup key file: {e}")

        self._auto_cert_path = None
        self._auto_key_path = None

    async def join_room(self, room_id: str, connection_id: str):
        """Add a connection to a room."""
        if room_id not in self.connection_rooms:
            self.connection_rooms[room_id] = set()

        self.connection_rooms[room_id].add(connection_id)
        self.room_connections[connection_id] = room_id

        logger.debug(f"Connection {connection_id} joined room {room_id}")

    async def leave_room(self, room_id: str, connection_id: str):
        """Remove a connection from a room."""
        if room_id in self.connection_rooms:
            self.connection_rooms[room_id].discard(connection_id)

        if connection_id in self.room_connections:
            if self.room_connections[connection_id] == room_id:
                del self.room_connections[connection_id]

        logger.debug(f"Connection {connection_id} left room {room_id}")

    async def leave_all_rooms(self, connection_id: str):
        """Remove a connection from all rooms."""
        if connection_id in self.room_connections:
            room_id = self.room_connections[connection_id]
            await self.leave_room(room_id, connection_id)

    async def broadcast_to_room(
        self,
        room_id: str,
        message: WebSocketMessage
    ):
        """
        Broadcast a message to all connections in a room.

        Args:
            room_id: Room identifier
            message: Message to broadcast
        """
        if room_id not in self.connection_rooms:
            return

        message.room = room_id
        encoded = WebSocketProtocol.encode(message)

        # Track broadcast duration for metrics
        start_time = time.time()

        # Send to all connections in the room
        for connection_id in list(self.connection_rooms[room_id]):
            if connection_id in self.connections:
                try:
                    websocket = self.connections[connection_id]
                    await websocket.send(encoded)
                except Exception as e:
                    logger.error(f"Error sending to {connection_id}: {e}")

        # Record broadcast metrics
        if self.metrics:
            duration = time.time() - start_time
            self.metrics.on_broadcast(room_id, duration)

    async def send_to_connection(
        self,
        connection_id: str,
        message: WebSocketMessage
    ):
        """
        Send a message to a specific connection.

        Args:
            connection_id: Connection identifier
            message: Message to send
        """
        if connection_id not in self.connections:
            logger.warning(f"Connection {connection_id} not found")
            return

        encoded = WebSocketProtocol.encode(message)

        try:
            websocket = self.connections[connection_id]
            await websocket.send(encoded)

            # Record sent message metric
            if self.metrics:
                self.metrics.on_message_sent(str(message.type))
        except Exception as e:
            logger.error(f"Error sending to {connection_id}: {e}")

            if self.metrics:
                self.metrics.on_message_error("send_failed")

    def on_event(self, event_type: str):
        """
        Decorator to register event handler.

        Usage:
            @server.on_event("session.updated")
            async def handle_session_update(data):
                print(f"Session updated: {data}")
        """
        def decorator(func: Callable) -> Callable:
            if event_type not in self.event_handlers:
                self.event_handlers[event_type] = set()
            self.event_handlers[event_type].add(func)
            return func
        return decorator

    async def emit_event(self, event_type: str, data: dict[str, Any]):
        """
        Emit an event to all registered handlers.

        Args:
            event_type: Event type
            data: Event data
        """
        if event_type not in self.event_handlers:
            return

        for handler in self.event_handlers[event_type]:
            try:
                import asyncio
                if asyncio.iscoroutinefunction(handler):
                    await handler(data)
                else:
                    handler(data)
            except Exception as e:
                logger.error(f"Error in event handler for {event_type}: {e}")


import logging



"""Backend implementations for prompting adapter."""

from mcp_common.backends.pyobjc import PyObjCPromptBackend
from mcp_common.backends.toolkit import PromptToolkitBackend

__all__ = ["PyObjCPromptBackend", "PromptToolkitBackend"]



#!/usr/bin/env python3
"""Demo script for prompting adapter.

This script demonstrates the unified prompting adapter with automatic backend detection.
Run with: python examples/prompt_demo.py
"""

import asyncio
from mcp_common.prompting import create_prompt_adapter, NotificationLevel


async def main():
    """Run prompting adapter demonstrations."""

    # Create adapter with auto-detection
    adapter = create_prompt_adapter()
    print(f"Using backend: {adapter.backend_name}\n")

    # 1. Simple alert
    print("=== 1. Alert Dialog ===")
    result = await adapter.alert(
        title="Welcome",
        message="This is a demonstration of the prompting adapter.",
        buttons=["Got it!", "Tell me more"],
        default_button="Got it!",
    )
    print(f"Button clicked: {result.button_clicked}\n")

    # 2. Confirmation dialog
    print("=== 2. Confirmation Dialog ===")
    confirmed = await adapter.confirm(
        title="Continue Demo",
        message="Would you like to see more examples?",
        default=True,
    )
    print(f"Confirmed: {confirmed}\n")

    if not confirmed:
        print("Demo cancelled by user.")
        return

    # 3. Text input
    print("=== 3. Text Input ===")
    name = await adapter.prompt_text(
        title="Personalization",
        message="What should we call you?",
        default="Claude User",
        placeholder="Enter your name",
    )
    print(f"Name: {name}\n")

    # 4. Choice selection
    print("=== 4. Choice Selection ===")
    choice = await adapter.prompt_choice(
        title="Favorite Color",
        message="What's your favorite color?",
        choices=["Red", "Green", "Blue", "Other"],
        default="Blue",
    )
    print(f"Selected: {choice}\n")

    # 5. Notification
    print("=== 5. Notification ===")
    await adapter.notify(
        title="Demo Complete",
        message="All demonstrations finished successfully!",
        level=NotificationLevel.SUCCESS,
        sound=True,
    )
    print("Notification sent!\n")

    # 6. File selection (optional, can skip in CI)
    print("=== 6. File Selection (Optional) ===")
    try:
        file_result = await adapter.select_file(
            title="Select a file to inspect",
            allowed_types=["py", "md", "txt"],
            multiple=False,
        )
        if file_result:
            print(f"Selected file: {file_result[0]}\n")
        else:
            print("No file selected (cancelled)\n")
    except Exception as e:
        print(f"File selection skipped: {e}\n")

    # 7. Directory selection (optional)
    print("=== 7. Directory Selection (Optional) ===")
    try:
        dir_result = await adapter.select_directory(
            title="Select a working directory",
        )
        if dir_result:
            print(f"Selected directory: {dir_result}\n")
        else:
            print("No directory selected (cancelled)\n")
    except Exception as e:
        print(f"Directory selection skipped: {e}\n")

    print("=== Demo Complete ===")
    print(f"Thanks, {name}!")


if __name__ == "__main__":
    asyncio.run(main())



"""Tests for LLM-Friendly API Response Schemas."""

from __future__ import annotations

import json

import pytest
from pydantic import ValidationError

from mcp_common.schemas import ToolInput, ToolResponse


@pytest.mark.unit
class TestToolResponse:
    """Tests for ToolResponse schema."""

    def test_tool_response_success_basic(self) -> None:
        """Test basic successful tool response."""
        response = ToolResponse(
            success=True,
            message="Operation completed successfully",
        )

        assert response.success is True
        assert response.message == "Operation completed successfully"
        assert response.data is None
        assert response.error is None
        assert response.next_steps is None

    def test_tool_response_with_data(self) -> None:
        """Test tool response with structured data."""
        response = ToolResponse(
            success=True,
            message="Created 3 user records",
            data={
                "records": [
                    {"id": "1", "name": "Alice"},
                    {"id": "2", "name": "Bob"},
                    {"id": "3", "name": "Charlie"},
                ]
            },
        )

        assert response.success is True
        assert len(response.data["records"]) == 3
        assert response.data["records"][0]["name"] == "Alice"

    def test_tool_response_with_next_steps(self) -> None:
        """Test tool response with suggested next steps."""
        response = ToolResponse(
            success=True,
            message="Records created",
            data={"count": 3},
            next_steps=[
                "Verify records in database",
                "Send confirmation emails",
                "Update cache",
            ],
        )

        assert response.success is True
        assert len(response.next_steps) == 3
        assert "Verify records" in response.next_steps[0]

    def test_tool_response_failure(self) -> None:
        """Test failed tool response."""
        response = ToolResponse(
            success=False,
            message="Failed to connect to database",
            error="Connection timeout after 30s - check database connectivity",
            next_steps=[
                "Check database status",
                "Verify network connectivity",
                "Retry with increased timeout",
            ],
        )

        assert response.success is False
        assert "Connection timeout" in response.error
        assert len(response.next_steps) == 3

    def test_tool_response_all_fields(self) -> None:
        """Test tool response with all fields populated."""
        response = ToolResponse(
            success=True,
            message="Operation completed",
            data={"result": "success", "count": 42},
            error=None,
            next_steps=["Step 1", "Step 2"],
        )

        assert response.success is True
        assert response.data["count"] == 42
        assert response.error is None
        assert response.next_steps == ["Step 1", "Step 2"]

    def test_tool_response_required_fields(self) -> None:
        """Test that success and message are required."""
        with pytest.raises(ValidationError) as exc_info:
            ToolResponse(success=True)  # Missing message

        errors = exc_info.value.errors()
        error_fields = {e["loc"][0] for e in errors}
        assert "message" in error_fields

    def test_tool_response_invalid_success_type(self) -> None:
        """Test that success must be boolean."""
        with pytest.raises(ValidationError) as exc_info:
            ToolResponse(
                success="yes",  # Should be bool
                message="Test",
            )

        errors = exc_info.value.errors()
        assert any(e["loc"][0] == "success" for e in errors)

    def test_tool_response_to_dict(self) -> None:
        """Test ToolResponse can be converted to dict."""
        response = ToolResponse(
            success=True,
            message="Test",
            data={"key": "value"},
        )

        response_dict = response.model_dump()

        assert response_dict["success"] is True
        assert response_dict["message"] == "Test"
        assert response_dict["data"]["key"] == "value"

    def test_tool_response_to_json(self) -> None:
        """Test ToolResponse can be serialized to JSON."""
        response = ToolResponse(
            success=True,
            message="Test",
            data={"count": 5},
        )

        json_str = response.model_dump_json()

        parsed = json.loads(json_str)
        assert parsed["success"] is True
        assert parsed["data"]["count"] == 5

    def test_tool_response_empty_data(self) -> None:
        """Test tool response with empty data dict."""
        response = ToolResponse(
            success=True,
            message="No results",
            data={},
        )

        assert response.data == {}

    def test_tool_response_complex_data(self) -> None:
        """Test tool response with nested complex data."""
        response = ToolResponse(
            success=True,
            message="Complex data",
            data={
                "users": [
                    {
                        "id": 1,
                        "name": "Alice",
                        "metadata": {"role": "admin", "active": True},
                    }
                ],
                "pagination": {"page": 1, "total": 100},
            },
        )

        assert response.data["users"][0]["metadata"]["role"] == "admin"
        assert response.data["pagination"]["total"] == 100


@pytest.mark.unit
class TestToolResponseExamples:
    """Tests using the example data from ToolResponse schema."""

    def test_example_1_success_case(self) -> None:
        """Test first example: successful user creation."""
        example = ToolResponse.model_json_schema()["examples"][0]

        response = ToolResponse(**example)

        assert response.success is True
        assert "Successfully created" in response.message
        assert len(response.data["records"]) == 3

    def test_example_2_failure_case(self) -> None:
        """Test second example: database connection failure."""
        example = ToolResponse.model_json_schema()["examples"][1]

        response = ToolResponse(**example)

        assert response.success is False
        assert "Failed to connect" in response.message
        assert response.error is not None


@pytest.mark.unit
class TestToolInput:
    """Tests for ToolInput schema."""

    def test_tool_input_basic(self) -> None:
        """Test basic tool input."""
        tool_input = ToolInput(
            name="search_users",
            description="Search for users by name or email",
            parameters={
                "type": "object",
                "properties": {
                    "query": {
                        "type": "string",
                        "description": "Search query (name or email)",
                    }
                },
                "required": ["query"],
            },
            example={"query": "alice@example.com"},
        )

        assert tool_input.name == "search_users"
        assert "Search for users" in tool_input.description

    def test_tool_input_to_example(self) -> None:
        """Test to_example method generates correct format."""
        tool_input = ToolInput(
            name="send_email",
            description="Send transactional email",
            parameters={
                "type": "object",
                "properties": {
                    "to": {"type": "string", "description": "Recipient email"},
                    "subject": {"type": "string", "description": "Email subject"},
                },
                "required": ["to", "subject"],
            },
            example={"to": "user@example.com", "subject": "Test Email"},
        )

        example_dict = tool_input.to_example()

        assert "description" in example_dict
        assert "parameters" in example_dict
        assert "example" in example_dict
        assert example_dict["example"]["to"] == "user@example.com"

    def test_tool_input_complex_parameters(self) -> None:
        """Test tool input with complex parameter schema."""
        tool_input = ToolInput(
            name="advanced_search",
            description="Advanced search with filters",
            parameters={
                "type": "object",
                "properties": {
                    "query": {"type": "string"},
                    "filters": {
                        "type": "object",
                        "properties": {
                            "date_range": {"type": "string"},
                            "category": {"type": "string"},
                        },
                    },
                    "limit": {"type": "integer", "minimum": 1, "maximum": 100},
                },
                "required": ["query"],
            },
            example={"query": "test", "limit": 10},
        )

        assert tool_input.parameters["properties"]["filters"]["type"] == "object"

    def test_tool_input_required_fields(self) -> None:
        """Test that all ToolInput fields are required."""
        with pytest.raises(ValidationError) as exc_info:
            ToolInput(
                name="test",
                # Missing description, parameters, example
            )

        errors = exc_info.value.errors()
        error_fields = {e["loc"][0] for e in errors}
        assert "description" in error_fields
        assert "parameters" in error_fields
        assert "example" in error_fields

    def test_tool_input_name_convention(self) -> None:
        """Test tool input names follow snake_case convention."""
        tool_input = ToolInput(
            name="get_user_profile",
            description="Get user profile",
            parameters={"type": "object", "properties": {}, "required": []},
            example={},
        )

        assert "_" in tool_input.name
        assert " " not in tool_input.name


@pytest.mark.unit
class TestToolResponseEncoding:
    """Tests for custom encoding in schemas."""

    def test_tool_response_with_set_data(self) -> None:
        """Test that sets in data are encoded as lists."""
        # Create response with set (will be converted to list)
        response = ToolResponse(
            success=True,
            message="Test",
            data={"tags": {1, 2, 3}},  # Set will be preserved
        )

        # Model should handle set encoding
        assert isinstance(response.data["tags"], set)

    def test_tool_input_with_set_encoding(self) -> None:
        """Test that ToolInput handles set encoding."""
        tool_input = ToolInput(
            name="test",
            description="Test",
            parameters={
                "type": "object",
                "properties": {},
                "required": [],
            },
            example={"tags": {1, 2, 3}},
        )

        # Should handle set in example
        assert "tags" in tool_input.example


@pytest.mark.unit
class TestToolResponseValidation:
    """Tests for ToolResponse validation patterns."""

    def test_success_must_be_bool(self) -> None:
        """Test success field only accepts boolean values."""
        # Valid booleans
        ToolResponse(success=True, message="Test")
        ToolResponse(success=False, message="Test")

        # Invalid: string
        with pytest.raises(ValidationError):
            ToolResponse(success="true", message="Test")

        # Invalid: int
        with pytest.raises(ValidationError):
            ToolResponse(success=1, message="Test")

    def test_message_required(self) -> None:
        """Test message field is always required."""
        with pytest.raises(ValidationError):
            ToolResponse(success=True)  # Missing message

        with pytest.raises(ValidationError):
            ToolResponse(success=True, message="")  # Empty message

    def test_optional_fields_can_be_omitted(self) -> None:
        """Test that optional fields can be None or omitted."""
        # All valid
        ToolResponse(
            success=True,
            message="Test",
            data=None,
            error=None,
            next_steps=None,
        )

    def test_next_steps_must_be_list(self) -> None:
        """Test next_steps must be a list of strings."""
        # Valid list
        ToolResponse(
            success=True,
            message="Test",
            next_steps=["Step 1", "Step 2"],
        )

        # Invalid: not a list
        with pytest.raises(ValidationError):
            ToolResponse(
                success=True,
                message="Test",
                next_steps="Step 1",  # Should be list
            )


@pytest.mark.unit
class TestToolInputValidation:
    """Tests for ToolInput validation patterns."""

    def test_parameters_must_be_dict(self) -> None:
        """Test parameters field must be a dictionary."""
        ToolInput(
            name="test",
            description="Test",
            parameters={},
            example={},
        )

        with pytest.raises(ValidationError):
            ToolInput(
                name="test",
                description="Test",
                parameters="not a dict",  # type: ignore
                example={},
            )

    def test_example_must_be_dict(self) -> None:
        """Test example field must be a dictionary."""
        ToolInput(
            name="test",
            description="Test",
            parameters={},
            example={},
        )

        with pytest.raises(ValidationError):
            ToolInput(
                name="test",
                description="Test",
                parameters={},
                example="not a dict",  # type: ignore
            )


@pytest.mark.unit
class TestToolResponseRealWorldScenarios:
    """Tests mimicking real-world MCP tool usage."""

    def test_database_query_response(self) -> None:
        """Test response from database query tool."""
        response = ToolResponse(
            success=True,
            message="Retrieved 5 user records",
            data={
                "records": [
                    {"id": 1, "name": "Alice", "email": "alice@example.com"},
                    {"id": 2, "name": "Bob", "email": "bob@example.com"},
                    {"id": 3, "name": "Charlie", "email": "charlie@example.com"},
                    {"id": 4, "name": "Diana", "email": "diana@example.com"},
                    {"id": 5, "name": "Eve", "email": "eve@example.com"},
                ],
                "total": 5,
                "query_time_ms": 23,
            },
            next_steps=[
                "Process user records",
                "Send notifications if needed",
            ],
        )

        assert response.success is True
        assert len(response.data["records"]) == 5
        assert response.data["query_time_ms"] == 23

    def test_api_request_failure_response(self) -> None:
        """Test response from failed API request."""
        response = ToolResponse(
            success=False,
            message="External API request failed",
            error="HTTP 400 Bad Request: Invalid parameter 'limit' must be >= 1",
            data=None,
            next_steps=[
                "Validate request parameters",
                "Check API documentation",
                "Retry with corrected parameters",
            ],
        )

        assert response.success is False
        assert "HTTP 400" in response.error
        assert len(response.next_steps) == 3

    def test_file_operation_response(self) -> None:
        """Test response from file operation tool."""
        response = ToolResponse(
            success=True,
            message="File copied successfully",
            data={
                "source": "/path/to/source.txt",
                "destination": "/path/to/destination.txt",
                "bytes_copied": 1024,
                "duration_seconds": 0.5,
            },
        )

        assert response.success is True
        assert response.data["bytes_copied"] == 1024

    def test_validation_error_response(self) -> None:
        """Test response for validation error."""
        response = ToolResponse(
            success=False,
            message="Input validation failed",
            error="Email address 'invalid-email' is not a valid format",
            data={
                "field": "email",
                "value": "invalid-email",
                "constraint": "must match email regex pattern",
            },
            next_steps=[
                "Correct email format",
                "Resubmit with valid email",
            ],
        )

        assert response.success is False
        assert "not a valid format" in response.error



"""Benchmarks for configuration loading performance."""

import pytest
from pathlib import Path
from mcp_common.config import MCPBaseSettings


class TestConfigBenchmarks:
    """Benchmark configuration loading operations."""

    @pytest.mark.benchmark(group="config", min_rounds=10)
    def test_yaml_loading_default(self, benchmark):
        """Benchmark YAML loading with default values only."""

        def load():
            class TestSettings(MCPBaseSettings):
                test_field: str = "default"

            return TestSettings.load("benchmark-test")

        result = benchmark(load)
        assert result.test_field == "default"

    @pytest.mark.benchmark(group="config", min_rounds=10)
    def test_yaml_loading_with_file(self, benchmark, tmp_path):
        """Benchmark YAML loading from file."""
        import yaml

        # Create settings directory in temp path
        settings_dir = tmp_path / "settings"
        settings_dir.mkdir(parents=True, exist_ok=True)

        yaml_file = settings_dir / "benchmark-test.yaml"
        yaml_file.write_text("test_field: from_file\n")

        def load():
            class TestSettings(MCPBaseSettings):
                test_field: str = "default"

            # Load from the temp path
            import os
            old_cwd = os.getcwd()
            try:
                os.chdir(str(tmp_path))
                return TestSettings.load("benchmark-test")
            finally:
                os.chdir(old_cwd)

        result = benchmark(load)
        assert result.test_field == "from_file"

    @pytest.mark.benchmark(group="config", min_rounds=10)
    def test_env_var_override(self, benchmark, monkeypatch):
        """Benchmark environment variable override."""
        monkeypatch.setenv("BENCHMARK_TEST_TEST_FIELD", "from_env")

        def load():
            class TestSettings(MCPBaseSettings):
                test_field: str = "default"

            return TestSettings.load("benchmark-test")

        result = benchmark(load)
        assert result.test_field == "from_env"

    @pytest.mark.benchmark(group="config", min_rounds=10)
    def test_settings_validation(self, benchmark):
        """Benchmark settings validation with Pydantic."""

        def load():
            class TestSettings(MCPBaseSettings):
                pass  # No custom fields to test base validation

            return TestSettings.load("benchmark-test")

        result = benchmark(load)
        # Just verify it loads without error
        assert result is not None



"""Tests for PromptBackend abstract base class."""

from __future__ import annotations

import inspect

import pytest

from mcp_common.prompting.base import PromptBackend
from mcp_common.prompting.models import DialogResult, NotificationLevel, PromptAdapterSettings


@pytest.mark.unit
class TestPromptBackendInterface:
    """Tests for PromptBackend abstract interface."""

    def test_prompt_backend_is_abstract(self) -> None:
        """Test PromptBackend cannot be instantiated directly."""
        with pytest.raises(TypeError):
            PromptBackend()  # type: ignore

    def test_prompt_backend_has_abstract_methods(self) -> None:
        """Test PromptBackend defines required abstract methods."""
        abstract_methods = PromptBackend.__abstractmethods__

        required_methods = {
            "alert",
            "confirm",
            "prompt_text",
            "prompt_choice",
            "notify",
            "select_file",
            "select_directory",
            "is_available",
        }

        assert required_methods.issubset(abstract_methods)

    def test_prompt_backend_method_signatures(self) -> None:
        """Test PromptBackend method signatures are correct."""
        # Check alert method
        alert_sig = inspect.signature(PromptBackend.alert)
        assert "title" in alert_sig.parameters
        assert "message" in alert_sig.parameters
        assert "detail" in alert_sig.parameters
        assert "buttons" in alert_sig.parameters
        assert "default_button" in alert_sig.parameters
        assert "style" in alert_sig.parameters

        # Check confirm method
        confirm_sig = inspect.signature(PromptBackend.confirm)
        assert "title" in confirm_sig.parameters
        assert "message" in confirm_sig.parameters
        assert "default" in confirm_sig.parameters
        assert "yes_label" in confirm_sig.parameters
        assert "no_label" in confirm_sig.parameters

        # Check prompt_text method
        prompt_text_sig = inspect.signature(PromptBackend.prompt_text)
        assert "title" in prompt_text_sig.parameters
        assert "message" in prompt_text_sig.parameters
        assert "default" in prompt_text_sig.parameters
        assert "placeholder" in prompt_text_sig.parameters
        assert "secure" in prompt_text_sig.parameters

        # Check notify method
        notify_sig = inspect.signature(PromptBackend.notify)
        assert "title" in notify_sig.parameters
        assert "message" in notify_sig.parameters
        assert "level" in notify_sig.parameters
        assert "sound" in notify_sig.parameters

        # Check select_file method
        select_file_sig = inspect.signature(PromptBackend.select_file)
        assert "title" in select_file_sig.parameters
        assert "allowed_types" in select_file_sig.parameters
        assert "multiple" in select_file_sig.parameters

        # Check select_directory method
        select_dir_sig = inspect.signature(PromptBackend.select_directory)
        assert "title" in select_dir_sig.parameters

    def test_prompt_backend_has_backend_name_property(self) -> None:
        """Test PromptBackend has backend_name as abstract property."""
        # backend_name should be a property, not a method
        assert isinstance(PromptBackend.backend_name, property)


@pytest.mark.unit
class TestPromptBackendCompliance:
    """Tests that concrete backends comply with PromptBackend interface."""

    @pytest.mark.asyncio
    async def test_mock_backend_implements_all_methods(self) -> None:
        """Test that a mock backend can implement all required methods."""

        class MockPromptBackend(PromptBackend):
            """Mock implementation for testing."""

            def __init__(self) -> None:
                self.backend_name_value = "mock"

            async def alert(
                self,
                title: str,
                message: str,
                detail: str | None = None,
                buttons: list[str] | None = None,
                default_button: str | None = None,
                style: str = "info",
            ) -> DialogResult:
                return DialogResult(button_clicked="OK")

            async def confirm(
                self,
                title: str,
                message: str,
                default: bool = False,
                yes_label: str = "Yes",
                no_label: str = "No",
            ) -> bool:
                return default

            async def prompt_text(
                self,
                title: str,
                message: str,
                default: str = "",
                placeholder: str = "",
                secure: bool = False,
            ) -> str | None:
                return default

            async def prompt_choice(
                self,
                title: str,
                message: str,
                choices: list[str],
                default: str | None = None,
            ) -> str | None:
                return default if default else (choices[0] if choices else None)

            async def notify(
                self,
                title: str,
                message: str,
                level: NotificationLevel = NotificationLevel.INFO,
                sound: bool = True,
            ) -> bool:
                return True

            async def select_file(
                self,
                title: str,
                allowed_types: list[str] | None = None,
                multiple: bool = False,
            ) -> list | None:
                return []

            async def select_directory(self, title: str) -> str | None:
                return None

            async def initialize(self) -> None:
                """Initialize mock backend (no-op)."""
                pass

            async def shutdown(self) -> None:
                """Cleanup mock backend (no-op)."""
                pass

            def is_available(self) -> bool:
                return True

            @property
            def backend_name(self) -> str:
                return self.backend_name_value

        # Test that mock backend can be instantiated and used
        backend = MockPromptBackend()
        assert backend.is_available() is True
        assert backend.backend_name == "mock"

        # Test all methods can be called
        result = await backend.alert("Test", "Message")
        assert result.button_clicked == "OK"

        result = await backend.confirm("Test", "Message")
        assert isinstance(result, bool)

        result = await backend.prompt_text("Test", "Message")
        assert isinstance(result, str | None)

        result = await backend.prompt_choice("Test", "Message", ["A", "B"])
        assert isinstance(result, str | None)

        result = await backend.notify("Test", "Message")
        assert result is True

        result = await backend.select_file("Test")
        assert isinstance(result, list | None)

        result = await backend.select_directory("Test")
        assert isinstance(result, str | None)


@pytest.mark.unit
class TestPromptBackendDocumentation:
    """Tests for PromptBackend documentation completeness."""

    def test_alert_method_has_docstring(self) -> None:
        """Test alert method has documentation."""
        assert PromptBackend.alert.__doc__ is not None
        assert len(PromptBackend.alert.__doc__) > 0

    def test_confirm_method_has_docstring(self) -> None:
        """Test confirm method has documentation."""
        assert PromptBackend.confirm.__doc__ is not None
        assert len(PromptBackend.confirm.__doc__) > 0

    def test_prompt_text_method_has_docstring(self) -> None:
        """Test prompt_text method has documentation."""
        assert PromptBackend.prompt_text.__doc__ is not None
        assert len(PromptBackend.prompt_text.__doc__) > 0

    def test_notify_method_has_docstring(self) -> None:
        """Test notify method has documentation."""
        assert PromptBackend.notify.__doc__ is not None
        assert len(PromptBackend.notify.__doc__) > 0

    def test_select_file_method_has_docstring(self) -> None:
        """Test select_file method has documentation."""
        assert PromptBackend.select_file.__doc__ is not None
        assert len(PromptBackend.select_file.__doc__) > 0

    def test_select_directory_method_has_docstring(self) -> None:
        """Test select_directory method has documentation."""
        assert PromptBackend.select_directory.__doc__ is not None
        assert len(PromptBackend.select_directory.__doc__) > 0



"""Usage profiles for MCP servers.

This module provides pre-configured server profiles for different use cases:

- **MinimalServer**: Basic tools only (simplest, fastest)
- **StandardServer**: Tools + resources (most common)
- **FullServer**: All features including auth, telemetry (production-ready)

Usage:
    >>> from mcp_common.profiles import MinimalServer, StandardServer, FullServer
    >>>
    >>> # Minimal server
    >>> server = MinimalServer(name="minimal-server")
    >>>
    >>> # Standard server
    >>> server = StandardServer(name="standard-server")
    >>>
    >>> # Full server with auth and telemetry
    >>> server = FullServer(
    ...     name="full-server",
    ...     auth=JWTAuth(secret="secret"),
    ...     telemetry=OpenTelemetry(endpoint="http://jaeger:4317")
    ... )
"""

from __future__ import annotations

from mcp_common.profiles.full import FullServer
from mcp_common.profiles.minimal import MinimalServer
from mcp_common.profiles.standard import StandardServer

__all__ = [
    "MinimalServer",
    "StandardServer",
    "FullServer",
]
